#include <stdlib.h>
#include <be/app/Message.h>
#include <be/app/Messenger.h>
#include <be/interface/Box.h>
#include <be/interface/CheckBox.h>
#include <be/interface/Menu.h>
#include <be/interface/MenuField.h>
#include <be/interface/MenuItem.h>
#include <be/interface/OptionPopUp.h>
#include <be/interface/RadioButton.h>
#include <be/interface/Rect.h>
#include <be/interface/TextControl.h>
#include <be/storage/Entry.h>
#include <be/storage/File.h>
#include <be/storage/NodeInfo.h>
#include <be/storage/Path.h>
#include <be/support/Debug.h>
#include <be/support/List.h>
#include <EntryRefItem/EntryRefItem.h>
#include <AudioInfo/AudioAttributes.h>
#include <AudioInfo/GenreList.h>
#include <AudioInfo/ID3Tags.h>
#include "AppDefs.h"
#include "AppView.h"
#include "AppWindow.h"
#include "CommandConstants.h"
#include "EditorView.h"
#include "GUIStrings.h"

EditorView::EditorView(BRect frame) : AddOnView(frame,EDITOR_VIEW_NAME)
{
	PRINT(("EditorView::EditorView(BRect)\n"));

	InitView();
}

EditorView::~EditorView()
{
	PRINT(("EditorView::~EditorView()\n"));
}

void
EditorView::InitView()
{
	PRINT(("EditorView::InitView()\n"));

	int space = 6;

	BRect frame;

	BCheckBox cb(BRect(0,0,0,0),0,0,0);
	cb.ResizeToPreferred();

	attributeRadioButton = new BRadioButton(BRect(0,0,0,0),0,ATTRIBUTES_LABEL,
			new BMessage(RB_EVENT));
	attributeRadioButton->ResizeToPreferred();
	attributeRadioButton->SetValue(B_CONTROL_ON);

	tagRadioButton = new BRadioButton(BRect(0,0,0,0),0,TAGS_LABEL,
			new BMessage(RB_EVENT));
	tagRadioButton->ResizeToPreferred();

	applyCheckBox = new BCheckBox(BRect(0,0,0,0),0,APPLY_TO_ATTRIBUTES,0);
	applyCheckBox->ResizeToPreferred();
	applyCheckBox->SetLabel(APPLY_TO_TAGS);

	frame.left = space;
	frame.top = space;
	frame.right = applyCheckBox->Frame().Width() + (3 * space);

	if(attributeRadioButton->Frame().Height() >=
			tagRadioButton->Frame().Height())
	{
		frame.bottom = attributeRadioButton->Frame().Height() +
			applyCheckBox->Frame().Height() + (4 * space);
	}
	else
	{
		frame.bottom = tagRadioButton->Frame().Height() +
			applyCheckBox->Frame().Height() + (4 * space);
	}

	editBox = new BBox(frame,0);
	AddChild(editBox);
	editBox->AddChild(attributeRadioButton);
	editBox->AddChild(tagRadioButton);
	editBox->AddChild(applyCheckBox);
	attributeRadioButton->MoveBy(space,space);
	tagRadioButton->MoveBy(2*space+attributeRadioButton->Frame().Width(),space);
	applyCheckBox->MoveBy(space,2*space+attributeRadioButton->Frame().Height());
	
	artistCheckBox = new BCheckBox(BRect(0,0,0,0),0,ARTIST_LABEL,
			new BMessage(MSG_ARTIST_CB));
	artistCheckBox->ResizeToPreferred();
	frame = artistCheckBox->Frame();

	albumCheckBox = new BCheckBox(BRect(0,0,0,0),0,ALBUM_LABEL,
			new BMessage(MSG_ALBUM_CB));
	albumCheckBox->ResizeToPreferred();
	if(albumCheckBox->Frame().Width() > frame.Width())
	{
		frame = albumCheckBox->Frame();
	}

	titleCheckBox = new BCheckBox(BRect(0,0,0,0),0,TITLE_LABEL,
			new BMessage(MSG_TITLE_CB));
	titleCheckBox->ResizeToPreferred();
	if(titleCheckBox->Frame().Width() > frame.Width())
	{
		frame = titleCheckBox->Frame();
	}

	yearCheckBox = new BCheckBox(BRect(0,0,0,0),0,YEAR_LABEL,
			new BMessage(MSG_YEAR_CB));
	yearCheckBox->ResizeToPreferred();
	if(yearCheckBox->Frame().Width() > frame.Width())
	{
		frame = yearCheckBox->Frame();
	}

	commentCheckBox = new BCheckBox(BRect(0,0,0,0),0,COMMENT_LABEL,
			new BMessage(MSG_COMMENT_CB));
	commentCheckBox->ResizeToPreferred();
	if(commentCheckBox->Frame().Width() > frame.Width())
	{
		frame = commentCheckBox->Frame();
	}

	trackCheckBox = new BCheckBox(BRect(0,0,0,0),0,TRACK_LABEL,
			new BMessage(MSG_TRACK_CB));
	trackCheckBox->ResizeToPreferred();
	if(trackCheckBox->Frame().Width() > frame.Width())
	{
		frame = trackCheckBox->Frame();
	}

#ifdef _TTE_
	ratingCheckBox = new BCheckBox(BRect(0,0,0,0),0,RATING_LABEL,
			new BMessage(MSG_RATING_CB));
	ratingCheckBox->ResizeToPreferred();
	if(ratingCheckBox->Frame().Width() > frame.Width())
	{
		frame = ratingCheckBox->Frame();
	}

	tempoCheckBox = new BCheckBox(BRect(0,0,0,0),0,TEMPO_LABEL,
			new BMessage(MSG_TEMPO_CB));
	tempoCheckBox->ResizeToPreferred();
	if(tempoCheckBox->Frame().Width() > frame.Width())
	{
		frame = tempoCheckBox->Frame();
	}

	composerCheckBox = new BCheckBox(BRect(0,0,0,0),0,COMPOSER_LABEL,
			new BMessage(MSG_TEMPO_CB));
	composerCheckBox->ResizeToPreferred();
	if(composerCheckBox->Frame().Width() > frame.Width())
	{
		frame = composerCheckBox->Frame();
	}
#endif

	genreCheckBox = new BCheckBox(BRect(0,0,0,0),0,GENRE_LABEL,
			new BMessage(MSG_GENRE_CB));
	genreCheckBox->ResizeToPreferred();
	if(genreCheckBox->Frame().Width() > frame.Width())
	{
		frame = genreCheckBox->Frame();
	}

	float x,y;
	x = space;
	y = editBox->Frame().bottom + 2*space;
	artistCheckBox->MoveTo(x,y);
	y += frame.Height() + space;
	albumCheckBox->MoveTo(x,y);
	y += frame.Height() + space;
	titleCheckBox->MoveTo(x,y);
	y += frame.Height() + space;
	yearCheckBox->MoveTo(x,y);
	y += frame.Height() + space;
	commentCheckBox->MoveTo(x,y);
	y += frame.Height() + space;
	trackCheckBox->MoveTo(x,y);
	y += frame.Height() + space;
#ifdef _TTE_
	ratingCheckBox->MoveTo(x,y);
	y += frame.Height() + space;
	tempoCheckBox->MoveTo(x,y);
	y += frame.Height() + space;
	composerCheckBox->MoveTo(x,y);
	y += frame.Height() + space;
#endif
	genreCheckBox->MoveTo(x,y);

	BRect tcFrame(0,0,180,frame.Height());

	artistTextControl = new BTextControl(tcFrame,0,0,0,0);
	artistTextControl->MoveTo(frame.right+5,artistCheckBox->Frame().top);

	albumTextControl = new BTextControl(tcFrame,0,0,0,0);
	albumTextControl->MoveTo(frame.right+5,albumCheckBox->Frame().top);
	
	titleTextControl = new BTextControl(tcFrame,0,0,0,0);
	titleTextControl->MoveTo(frame.right+5,titleCheckBox->Frame().top);
	
	yearTextControl = new BTextControl(tcFrame,0,0,0,0);
	yearTextControl->MoveTo(frame.right+5,yearCheckBox->Frame().top);
	
	commentTextControl = new BTextControl(tcFrame,0,0,0,0);
	commentTextControl->MoveTo(frame.right+5,commentCheckBox->Frame().top);
	
	trackTextControl = new BTextControl(tcFrame,0,0,0,0);
	trackTextControl->MoveTo(frame.right+5,trackCheckBox->Frame().top);

#ifdef _TTE_
	ratingTextControl = new BTextControl(tcFrame,0,0,0,0);
	ratingTextControl->MoveTo(frame.right+5,ratingCheckBox->Frame().top);
	tempoTextControl = new BTextControl(tcFrame,0,0,0,0);
	tempoTextControl->MoveTo(frame.right+5,tempoCheckBox->Frame().top);
	composerTextControl = new BTextControl(tcFrame,0,0,0,0);
	composerTextControl->MoveTo(frame.right+5,composerCheckBox->Frame().top);
#endif

	//INSIDE BOX
	BMenu* menu = new BMenu(GENRE_LABEL);
	menu->SetLabelFromMarked(true);
	int genres = GenreList::NumGenres();
	BList genreList;
	for(int i=0;i<genres;i++)
	{
		char* genre = GenreList::Genre(i);
		genreList.AddItem(genre);

	}
	genreList.SortItems(GenreList::GenreSort);
	char last = 'A';
	char current;
	for(int i=0;i<genres;i++)
	{
		char* genre = (char*)genreList.ItemAt(i);
		current = genre[0];
		if(current != last)
		{
			menu->AddSeparatorItem();
		}
		menu->AddItem(new BMenuItem(genre,new BMessage(GENRE_SELECTION_CHANGED)));
		last = current;
	}

	genreMenuField = new BMenuField(BRect(space,space,0,0),0,0,menu);

	//since a BMenuField doesn't report it's adjusted size correctly
	//we use the height of a BTextControl to approximate the height
	//of the BMenuField.  Not perfect, but close.
	genreTextControl = new BTextControl(BRect(0,0,110-(2*space),0),0,0,0,0);
	genreTextControl->MoveTo(space,2*space+genreTextControl->Frame().bottom);
	//DONE INSIDE BOX

	tcFrame.bottom = genreTextControl->Frame().bottom + 2*space;
	genreBox = new BBox(tcFrame,0);
	genreBox->MoveTo(frame.right+5,genreCheckBox->Frame().top);

	ResizeToPreferred();

	PRINT_OBJECT(Frame());

	AddChild(artistCheckBox);
	AddChild(artistTextControl);

	AddChild(albumCheckBox);
	AddChild(albumTextControl);

	AddChild(titleCheckBox);
	AddChild(titleTextControl);

	AddChild(yearCheckBox);
	AddChild(yearTextControl);

	AddChild(commentCheckBox);
	AddChild(commentTextControl);

	AddChild(trackCheckBox);
	AddChild(trackTextControl);

#ifdef _TTE_
	AddChild(ratingCheckBox);
	AddChild(ratingTextControl);

	AddChild(tempoCheckBox);
	AddChild(tempoTextControl);

	AddChild(composerCheckBox);
	AddChild(composerTextControl);
#endif

	AddChild(genreCheckBox);
	AddChild(genreBox);
	genreBox->AddChild(genreMenuField);
	genreBox->AddChild(genreTextControl);

	WidgetsSetValues();
	WidgetsSetEnabled();
}

void
EditorView::AttachedToWindow()
{
	AddOnView::AttachedToWindow();

	PRINT(("EditorView::AttachedToWindow()\n"));

	tagRadioButton->SetTarget(this);
	attributeRadioButton->SetTarget(this);
	applyCheckBox->SetTarget(this);
	artistCheckBox->SetTarget(this);
	albumCheckBox->SetTarget(this);
	titleCheckBox->SetTarget(this);
	yearCheckBox->SetTarget(this);
	commentCheckBox->SetTarget(this);
	trackCheckBox->SetTarget(this);
#ifdef _TTE_
	ratingCheckBox->SetTarget(this);
	tempoCheckBox->SetTarget(this);
	composerCheckBox->SetTarget(this);
#endif
	genreCheckBox->SetTarget(this);
	genreMenuField->Menu()->SetTargetForItems(this);
}

void
EditorView::GetPreferredSize(float* width, float* height)
{
	PRINT(("EditorView::GetPreferredSize(float*,float*)\n"));

	int space = 6;

	*width = artistTextControl->Frame().right;
	if((editBox->Frame().right) > *width)
	{
		*width = editBox->Frame().right;
	}

	*height = genreBox->Frame().bottom;
}

void
EditorView::Hide()
{
	AddOnView::Hide();

	PRINT(("EditorView::Hide()\n"));
}

void
EditorView::Show()
{
	AddOnView::Show();

	PRINT(("EditorView::Show()\n"));
}

void
EditorView::SelectionChanged(BList* list)
{
	AddOnView::SelectionChanged(list);

	PRINT(("EditorView::SelectionChanged(BList* list)\n"));

	WidgetsSetValues();
	WidgetsSetEnabled();
}

void
EditorView::GenreSelectionAction()
{
	genreTextControl->SetText("");
}

void
EditorView::Apply()
{
	AddOnView::Apply();

	PRINT(("EditorView::Apply()\n"));

	thread_id thread = spawn_thread(EditorView::ApplyFunction,"TheEditor",
			B_NORMAL_PRIORITY,(void*)this);
	resume_thread(thread);
}

void
EditorView::Reset()
{
	AddOnView::Reset();

	PRINT(("EditorView::Reset()\n"));

	WidgetsSetValues();
	WidgetsSetEnabled();
	attributeRadioButton->SetValue(B_CONTROL_ON);
}

void
EditorView::SetEnabled(BCheckBox* checkbox, BControl* control)
{
	PRINT(("EditorView::SetEnabled(BCheckBox*,BControl*)\n"));

	if(checkbox->Value() == B_CONTROL_ON)
	{
		control->SetEnabled(true);
	}
	else
	{
		control->SetEnabled(false);
	}
}

void
EditorView::SetEnabled(BCheckBox* checkbox, BMenuField* menufield)
{
	PRINT(("EditorView::SetEnabled(BCheckBox*,BMenuField*)\n"));

	if(checkbox->Value() == B_CONTROL_ON)
	{
		menufield->SetEnabled(true);
	}
	else
	{
		menufield->SetEnabled(false);
	}
}

void
EditorView::WidgetsSetValues()
{
	PRINT(("EditorView::WidgetsSetValues()\n"));

	int numSelected = selectedItems->CountItems();
	if(numSelected == 0)
	{
		artistCheckBox->SetValue(B_CONTROL_OFF);
		albumCheckBox->SetValue(B_CONTROL_OFF);
		titleCheckBox->SetValue(B_CONTROL_OFF);
		commentCheckBox->SetValue(B_CONTROL_OFF);
		trackCheckBox->SetValue(B_CONTROL_OFF);
#ifdef _TTE_
		ratingCheckBox->SetValue(B_CONTROL_OFF);
		tempoCheckBox->SetValue(B_CONTROL_OFF);
		composerCheckBox->SetValue(B_CONTROL_OFF);
#endif
		yearCheckBox->SetValue(B_CONTROL_OFF);
		genreCheckBox->SetValue(B_CONTROL_OFF);
	}
	else if(numSelected == 1)
	{
		artistCheckBox->SetValue(B_CONTROL_ON);
		albumCheckBox->SetValue(B_CONTROL_ON);
		titleCheckBox->SetValue(B_CONTROL_ON);
		commentCheckBox->SetValue(B_CONTROL_ON);
		trackCheckBox->SetValue(B_CONTROL_ON);
#ifdef _TTE_
		ratingCheckBox->SetValue(B_CONTROL_ON);
		tempoCheckBox->SetValue(B_CONTROL_ON);
		composerCheckBox->SetValue(B_CONTROL_ON);
#endif
		yearCheckBox->SetValue(B_CONTROL_ON);
		genreCheckBox->SetValue(B_CONTROL_ON);
	}
	else if(numSelected > 1)
	{
		artistCheckBox->SetValue(B_CONTROL_ON);
		albumCheckBox->SetValue(B_CONTROL_ON);
		titleCheckBox->SetValue(B_CONTROL_OFF);
		commentCheckBox->SetValue(B_CONTROL_ON);
		trackCheckBox->SetValue(B_CONTROL_OFF);
#ifdef _TTE_
		ratingCheckBox->SetValue(B_CONTROL_ON);
		tempoCheckBox->SetValue(B_CONTROL_ON);
		composerCheckBox->SetValue(B_CONTROL_ON);
#endif
		yearCheckBox->SetValue(B_CONTROL_ON);
		genreCheckBox->SetValue(B_CONTROL_ON);
	}

	if(numSelected > 0)
	{
		EntryRefItem* item = (EntryRefItem*)selectedItems->ItemAt(0);
		entry_ref* ref = item->EntryRef();
		if(ref)
		{
			const char* artist;
			const char* album;
			const char* title;
			const char* year;
			const char* comment;
			const char* track;
#ifdef _TTE_
			const char* rating;
			const char* tempo;
			const char* composer;
#endif
			const char* genre;

			BFile audioFile(ref,B_READ_WRITE);
			BNodeInfo info(&audioFile);
			char type[B_MIME_TYPE_LENGTH];
			info.GetType(type);

			if(attributeRadioButton->Value() == B_CONTROL_ON)
			{
				AudioAttributes attributes(&audioFile);
				artist = attributes.Artist();
				if(!artist)
				{
					artist = "";
				}
				album = attributes.Album();
				if(!album)
				{
					album = "";
				}
				title = attributes.Title();
				if(!title)
				{
					title = "";
				}
				year = attributes.Year();
				if(!year)
				{
					year = "";
				}
				comment = attributes.Comment();
				if(!comment)
				{
					comment = "";
				}
				track = attributes.Track();
				if(!track)
				{
					track = "";
				}
#ifdef _TTE_
				rating = attributes.Rating();
				if(!rating)
				{
					rating = "";
				}

				tempo = attributes.Tempo();
				if(!tempo)
				{
					tempo = "";
				}

				composer = attributes.Composer();
				if(!composer)
				{
					composer = "";
				}
#endif
				genre = attributes.Genre();
				if(!genre)
				{
					genre = "";
				}

				artistTextControl->SetText(artist);
				albumTextControl->SetText(album);
				titleTextControl->SetText(title);
				yearTextControl->SetText(year);
				commentTextControl->SetText(comment);
				trackTextControl->SetText(track);
#ifdef _TTE_
				ratingTextControl->SetText(rating);
				tempoTextControl->SetText(tempo);
				composerTextControl->SetText(composer);
#endif
				BMenu* menu = genreMenuField->Menu();
				BMenuItem* item = menu->FindItem(genre);
				if(item)
				{
					item->SetMarked(true);
					genreTextControl->SetText("");
				}
				else
				{
					item = menu->FindItem("Other");
					item->SetMarked(true);
					genreTextControl->SetText(genre);
				}

				return;
			}
			else if(strcmp(type,MP3_MIME_TYPE) == 0)
			{
				BPath refPath(ref);
				ID3Tags tags(refPath.Path());

				artist = tags.Artist();
				if(!artist)
				{
					artist = "";
				}
				album = tags.Album();
				if(!album)
				{
					album = "";
				}
				title = tags.Title();
				if(!title)
				{
					title = "";
				}
				year = tags.Year();
				if(!year)
				{
					year = "";
				}
				comment = tags.Comment();
				if(!comment)
				{
					comment = "";
				}
				track = tags.Track();
				if(!track)
				{
					track = "";
				}
				genre = tags.Genre();
				if(!genre)
				{
					genre = "";
				}

				artistTextControl->SetText(artist);
				albumTextControl->SetText(album);
				titleTextControl->SetText(title);
				yearTextControl->SetText(year);
				commentTextControl->SetText(comment);
				trackTextControl->SetText(track);
#ifdef _TTE_
				ratingTextControl->SetText("");
				tempoTextControl->SetText("");
				composerTextControl->SetText("");
#endif
				BMenu* menu = genreMenuField->Menu();
				BMenuItem* item = menu->FindItem(genre);
				if(item)
				{
					item->SetMarked(true);
				}
				else
				{
					item = menu->FindItem("Other");
					item->SetMarked(true);
				}
				genreTextControl->SetText("");

				return;
			}
		}
	}

	artistTextControl->SetText("");
	albumTextControl->SetText("");
	titleTextControl->SetText("");
	yearTextControl->SetText("");
	commentTextControl->SetText("");
	trackTextControl->SetText("");
#ifdef _TTE_
	ratingTextControl->SetText("");
	tempoTextControl->SetText("");
	composerTextControl->SetText("");
#endif
	genreTextControl->SetText("");
}

void
EditorView::WidgetsSetEnabled()
{
	PRINT(("EditorView::WidgetsSetEnabled()\n"));

	int numSelected = selectedItems->CountItems();
	if(numSelected == 0)
	{
		attributeRadioButton->SetEnabled(false);
		tagRadioButton->SetEnabled(false);
		applyCheckBox->SetEnabled(false);
		artistCheckBox->SetEnabled(false);
		albumCheckBox->SetEnabled(false);
		titleCheckBox->SetEnabled(false);
		yearCheckBox->SetEnabled(false);
		commentCheckBox->SetEnabled(false);
		trackCheckBox->SetEnabled(false);
#ifdef _TTE_
		ratingCheckBox->SetEnabled(false);
		tempoCheckBox->SetEnabled(false);
		composerCheckBox->SetEnabled(false);
#endif
		genreCheckBox->SetEnabled(false);
		artistTextControl->SetEnabled(false);
		albumTextControl->SetEnabled(false);
		titleTextControl->SetEnabled(false);
		yearTextControl->SetEnabled(false);
		commentTextControl->SetEnabled(false);
		trackTextControl->SetEnabled(false);
#ifdef _TTE_
		ratingTextControl->SetEnabled(false);
		tempoTextControl->SetEnabled(false);
		composerTextControl->SetEnabled(false);
#endif
		genreMenuField->SetEnabled(false);
		genreTextControl->SetEnabled(false);
	}
	else if(numSelected > 0)
	{
		attributeRadioButton->SetEnabled(true);
		tagRadioButton->SetEnabled(true);
		applyCheckBox->SetEnabled(true);
		artistCheckBox->SetEnabled(true);
		albumCheckBox->SetEnabled(true);
		titleCheckBox->SetEnabled(true);
		yearCheckBox->SetEnabled(true);
		commentCheckBox->SetEnabled(true);
		trackCheckBox->SetEnabled(true);
		genreCheckBox->SetEnabled(true);
		artistTextControl->SetEnabled(true);
		albumTextControl->SetEnabled(true);
		titleTextControl->SetEnabled(true);
		yearTextControl->SetEnabled(true);
		commentTextControl->SetEnabled(true);
		trackTextControl->SetEnabled(true);
		genreMenuField->SetEnabled(true);
		genreTextControl->SetEnabled(true);

#ifdef _TTE_
		if(attributeRadioButton->Value() == B_CONTROL_ON)
		{
			ratingCheckBox->SetEnabled(true);
			ratingTextControl->SetEnabled(true);

			tempoCheckBox->SetEnabled(true);
			tempoTextControl->SetEnabled(true);

			composerCheckBox->SetEnabled(true);
			composerTextControl->SetEnabled(true);
		}
		else
		{
			ratingCheckBox->SetValue(B_CONTROL_OFF);
			ratingCheckBox->SetEnabled(false);
			ratingTextControl->SetEnabled(false);

			tempoCheckBox->SetValue(B_CONTROL_OFF);
			tempoCheckBox->SetEnabled(false);
			tempoTextControl->SetEnabled(false);

			composerCheckBox->SetValue(B_CONTROL_OFF);
			composerCheckBox->SetEnabled(false);
			composerTextControl->SetEnabled(false);
		}
#endif

		SetEnabled(artistCheckBox,artistTextControl);
		SetEnabled(albumCheckBox,albumTextControl);
		SetEnabled(titleCheckBox,titleTextControl);
		SetEnabled(yearCheckBox,yearTextControl);
		SetEnabled(commentCheckBox,commentTextControl);
		SetEnabled(trackCheckBox,trackTextControl);
#ifdef _TTE_
		SetEnabled(ratingCheckBox,ratingTextControl);
		SetEnabled(tempoCheckBox,tempoTextControl);
		SetEnabled(composerCheckBox,composerTextControl);
#endif
		SetEnabled(genreCheckBox,genreMenuField);
		SetEnabled(genreCheckBox,genreTextControl);
	}
}

void
EditorView::WidgetsRBValues()
{
	if(attributeRadioButton->Value() == B_CONTROL_ON)
	{
		applyCheckBox->SetLabel(APPLY_TO_TAGS);
		applyCheckBox->SetValue(B_CONTROL_OFF);
	}
	else
	{
		applyCheckBox->SetLabel(APPLY_TO_ATTRIBUTES);
		applyCheckBox->SetValue(B_CONTROL_OFF);
	}
}

int32
EditorView::ApplyFunction(void* args)
{
	PRINT(("EditorView::ApplyFunction(void*)\n"));

	EditorView* view = (EditorView*)args;

	BView* appView = view->Window()->FindView("AppView");
	BMessenger messenger(appView);
	BMessage startMsg(START_APPLY);
	BMessage endMsg(END_APPLY);
	BMessage maxMsg(STATUS_BAR_SET_MAX_VALUE);
	BMessage updateMsg(STATUS_BAR_UPDATE);
	BMessage resetMsg(STATUS_BAR_RESET);

	messenger.SendMessage(&startMsg);

	char artist[256];
	char album[256];
	char title[256];
	char year[256];
	char comment[256];
	char track[256];
#ifdef _TTE_
	char rating[256];
	char tempo[256];
	char composer[256];
#endif
	char genre[256];
	bool doAttributes = false;
	bool doTags = false;

	strcpy(artist,view->artistTextControl->Text());
	strcpy(album,view->albumTextControl->Text());
	strcpy(title,view->titleTextControl->Text());
	strcpy(year,view->yearTextControl->Text());
	strcpy(comment,view->commentTextControl->Text());
	strcpy(track,view->trackTextControl->Text());
#ifdef _TTE_
	strcpy(rating,view->ratingTextControl->Text());
	strcpy(tempo,view->tempoTextControl->Text());
	strcpy(composer,view->composerTextControl->Text());
#endif
	strcpy(genre,view->genreTextControl->Text());
	if(strcmp(genre,"") == 0)
	{
		BMenu* menu = view->genreMenuField->Menu();
		BMenuItem* item = menu->FindMarked();
		strcpy(genre,item->Label());
	}

	if((view->attributeRadioButton->Value() == B_CONTROL_ON) ||
			(view->applyCheckBox->Value() == B_CONTROL_ON))
	{
		doAttributes = true;
	}

	if((view->tagRadioButton->Value() == B_CONTROL_ON) ||
			(view->applyCheckBox->Value() == B_CONTROL_ON))
	{
		doTags = true;
	}

	int numSelected = view->selectedItems->CountItems();
	maxMsg.AddInt32("maxValue",numSelected);
	messenger.SendMessage(&maxMsg);
	for(int i=0;i<numSelected;i++)
	{
		EntryRefItem* refItem = (EntryRefItem*)view->selectedItems->ItemAt(i);
		entry_ref* ref = refItem->EntryRef();
		BFile audioFile(ref,B_READ_WRITE);
		BNodeInfo info(&audioFile);
		char type[B_MIME_TYPE_LENGTH];
		info.GetType(type);

		if(doAttributes)
		{
			AudioAttributes attributes(&audioFile);
			if(view->artistCheckBox->Value() == B_CONTROL_ON)
			{
				attributes.SetArtist(artist);
				attributes.WriteArtist();
			}
			if(view->albumCheckBox->Value() == B_CONTROL_ON)
			{
				attributes.SetAlbum(album);
				attributes.WriteAlbum();
			}
			if(view->titleCheckBox->Value() == B_CONTROL_ON)
			{
				attributes.SetTitle(title);
				attributes.WriteTitle();
			}
			if(view->yearCheckBox->Value() == B_CONTROL_ON)
			{
				attributes.SetYear(year);
				attributes.WriteYear();
			}
			if(view->commentCheckBox->Value() == B_CONTROL_ON)
			{
				attributes.SetComment(comment);
				attributes.WriteComment();
			}
			if(view->trackCheckBox->Value() == B_CONTROL_ON)
			{
				attributes.SetTrack(track);
				attributes.WriteTrack();
			}
#ifdef _TTE_
			if(view->ratingCheckBox->Value() == B_CONTROL_ON)
			{
				attributes.SetRating(rating);
				attributes.WriteRating();
			}
			if(view->tempoCheckBox->Value() == B_CONTROL_ON)
			{
				attributes.SetTempo(tempo);
				attributes.WriteTempo();
			}
			if(view->composerCheckBox->Value() == B_CONTROL_ON)
			{
				attributes.SetComposer(composer);
				attributes.WriteComposer();
			}
#endif
			if(view->genreCheckBox->Value() == B_CONTROL_ON)
			{
				attributes.SetGenre(genre);
				attributes.WriteGenre();
			}
		}

		if(doTags && (strcmp(type,MP3_MIME_TYPE) == 0))
		{
			BPath refPath(ref);
			ID3Tags tags(refPath.Path());
			if(view->artistCheckBox->Value() == B_CONTROL_ON)
			{
				tags.SetArtist(artist);
				tags.WriteArtist();
			}
			if(view->albumCheckBox->Value() == B_CONTROL_ON)
			{
				tags.SetAlbum(album);
				tags.WriteAlbum();
			}
			if(view->titleCheckBox->Value() == B_CONTROL_ON)
			{
				tags.SetTitle(title);
				tags.WriteTitle();
			}
			if(view->yearCheckBox->Value() == B_CONTROL_ON)
			{
				tags.SetYear(year);
				tags.WriteYear();
			}
			if(view->commentCheckBox->Value() == B_CONTROL_ON)
			{
				tags.SetComment(comment);
				tags.WriteComment();
			}
			if(view->trackCheckBox->Value() == B_CONTROL_ON)
			{
				tags.SetTrack(track);
				tags.WriteTrack();
			}
			if(view->genreCheckBox->Value() == B_CONTROL_ON)
			{
				tags.SetGenre(genre);
				tags.WriteGenre();
			}
		}
		messenger.SendMessage(&updateMsg);
	}
	messenger.SendMessage(&resetMsg);
	messenger.SendMessage(&endMsg);
}

void
EditorView::MessageReceived(BMessage* message)
{
	//PRINT(("EditorView::MessageReceived(BMessage*)\n"));

	switch(message->what)
	{
		case MSG_ARTIST_CB:
			SetEnabled(artistCheckBox,artistTextControl);
			break;
		case MSG_ALBUM_CB:
			SetEnabled(albumCheckBox,albumTextControl);
			break;
		case MSG_TITLE_CB:
			SetEnabled(titleCheckBox,titleTextControl);
			break;
		case MSG_YEAR_CB:
			SetEnabled(yearCheckBox,yearTextControl);
			break;
		case MSG_COMMENT_CB:
			SetEnabled(commentCheckBox,commentTextControl);
			break;
		case MSG_TRACK_CB:
			SetEnabled(trackCheckBox,trackTextControl);
			break;
#ifdef _TTE_
		case MSG_RATING_CB:
			SetEnabled(ratingCheckBox,ratingTextControl);
			break;
		case MSG_TEMPO_CB:
			SetEnabled(tempoCheckBox,tempoTextControl);
			break;
		case MSG_COMPOSER_CB:
			SetEnabled(composerCheckBox,composerTextControl);
			break;
#endif
		case MSG_GENRE_CB:
			SetEnabled(genreCheckBox,genreMenuField);
			SetEnabled(genreCheckBox,genreTextControl);
			break;
		case RB_EVENT:
			WidgetsSetValues();
			WidgetsSetEnabled();
			WidgetsRBValues();
			break;
		case GENRE_SELECTION_CHANGED:
			GenreSelectionAction();
			break;
		default:
			AddOnView::MessageReceived(message);
	}
}
